#!/usr/bin/python3 
""" Application to Record and Transfer Clinical Therapy Video """ 
 
from flask import Flask, jsonify, render_template, request, url_for, redirect, send_from_directory 
from threading import Thread 
from time import sleep 
from tools import util 
from mpbc import info 
import json 
import os 
 
 
app = Flask(__name__) 
recording = util.Recording() 
default = info.Default() 
session = info.Session() 
device = info.Device() 
pin = util.PIN(device.PIN)
usb = util.USB()
ssids = util.get_network_ssids() 
SITE = default.SITE 
connected=False
upload_status=False
 
 
def authorize(func): 
    def wrapper(*args, **kwargs): 
        print('Authorized') 
        if not session.is_authorized(): 
            return render_pin() 
        return func(*args, **kwargs) 
    return wrapper 
 
 
def config(func): 
    def wrapper(*args, **kwargs): 
        if device.is_configured(): 
            template = render_pin() 
        else: 
            template = config.next() 
        return func(*args, **kwargs) 
    return wrapper 
 
 
@authorize 
def render_index(): 
    """ 
    Renders the index page with the default parameters 
    """ 
    recording = util.Recording() 
    default = info.Default() 
    return render_template( 
        'index.html', 
        sidebar = device.get_sidebar(), 
        name = 'record', 
        studies = info.Program.SITES[default.SITE][1], 
        visits = info.Program.VISITS, 
        participants = info.Program.PARTICIPANTS, 
        study = default.STUDY, 
        participant = default.PARTICIPANT, 
        visit = default.VISIT, 
        webcams = recording.webcams, 
        microphones = recording.microphones, 
    ) 
 
 
def render_pin(incorrect_pin=False): 
    """ 
    Renders the PIN entry page 
    """ 
    if session.is_authorized(): 
        return render_index() 
    else: 
        return render_template( 
            'enter_pin.html', 
            incorrect_pin = incorrect_pin, 
            remaining_attempts = pin.attempt_counter 
        ) 
 
 
#@app.after_request 
#def add_header(r): 
#    """ 
#    Add headers to both force latest IE rendering engine or Chrome Frame, 
#    and also to cache the rendered page for 10 minutes. 
#    """ 
#    r.headers["Cache-Control"] = "no-cache, no-store, must-revalidate, post-check=0, pre-check=0'" 
#    r.headers["Pragma"] = "no-cache" 
#    r.headers["Expires"] = "0" 
#    r.headers['Cache-Control'] = 'public, max-age=86400' 
#    return r 
 
 
@app.route('/favicon.ico', methods = ['GET'])  
def favicon():  
    return send_from_directory(os.path.join(app.root_path, 'static'), 'favicon.ico', mimetype='image/vnd.microsoft.icon') 
 
@app.route('/')
def index():
    return render_index(); 
#@app.route('/')
#def lock():
#    return render_template(
#       'usb_key.html',
#        attached = usb.is_storage_device_attached()
#    )


@app.route('/wait_for_usb_drive')
def wait():
    device_name = usb.monitor_devices()
    usb.mount(device_name)
    usb.decrypt()
    return jsonify(usb.data)


@app.route('/home')
def home():
    return render_index()


@app.route('/check_config')
def check_config():
    session.deauthorize() 
    device = info.Device() 
    if device.is_configured(): 
        return render_pin() 
    elif device.CSTATUS == 'factory': 
        return render_template('welcome.html') 
    else: 
        return render_template( 
            'resume_config.html', 
            resume_from = device.CSTATUS 
        ) 
 
 
@app.route('/check_pin', methods = ['GET']) 
def check_pin(): 
    entered_pin = request.args.get('entered_pin') 
    pin.check_pin(entered_pin) 
    return '200' 
 
 
@app.route('/setup_pin', methods = ['GET']) 
def setup_pin(): 
    entered_pin = request.args.get('entered_pin') 
    device.write_pin(entered_pin) 
    return '200' 
 
 
@app.route('/confirm_pin', methods = ['GET']) 
def confirm_pin(): 
    entered_pin = request.args.get('entered_pin') 
    device.compare_pins(entered_pin) 
    return '200' 
 
 
@app.route('/return_pin', methods = ['GET']) 
def return_pin(): 
    session.deauthorize() 
    if pin.attempt_counter < 1: 
        # notify sponsor 
        return render_template('error.html') 
    if pin.correct_pin: 
        session.authorize() 
    return render_pin(True) 
 
 
@app.route('/complete_pin', methods = ['GET']) 
def complete_pin(): 
    return render_template( 
        'confirm_pin.html', 
        sidebar = device.get_sidebar('config') 
    ) 
 
 
@app.route('/matched_pins', methods = ['GET']) 
def matched_pins(): 
    if device.pins_match: 
        if device.is_configured(): 
            return render_template( 
                'config_menu.html' 
            ) 
        else: 
            return render_template( 
                'successfully_configured.html' 
            ) 
    else: 
        return render_template( 
            'configure_pin.html', 
            name = 'pin', 
            sidebar = device.get_sidebar('config') 
        ) 
 
 
@authorize 
@app.route('/navigation', methods=['GET']) 
def navigation(): 
    btn = request.args.get('btn') 
    if btn == 'continue': 
        return render_index() 
    elif btn == 'back': 
        return render_pin() 
 
 
@app.route('/start_configuration', methods=['GET']) 
def start_configuration(): 
    device.toggle_sbar('open') 
    return render_template( 
        'select_network.html', 
        name = 'network', 
        ssids = util.get_network_ssids(), 
        sidebar = device.get_sidebar('config') 
    ) 
 
 
@app.route('/complete_configuration', methods=['GET']) 
def complete_configuration(): 
    device.toggle_sbar('closed') 
    device.configure() 
    print(device.PIN) 
    pin = util.PIN(device.PIN) 
    return render_index() 
 
 
@authorize 
@app.route('/start_recording', methods=['POST', 'GET']) 
def start_recording(): 
    recording = util.Recording() 
    btn = request.args.get('btn') 
    participant = request.args.get('participant') 
    config = [ 
        request.args.get('study'), 
        SITE, 
        participant, 
        request.args.get('visit'), 
        util.get_curtime(), 
        str(len(recording.webcams)), 
        str(len(recording.microphones)) 
    ] 
    config += [str(x) for x in recording.webcams] 
    config += [str(x) for x in recording.microphones] 
 
    default.write(config)  
 
    if participant in info.Program.PARTICIPANTS: 
        if btn == 'start': 
            recording.start() 
            return render_template('initialize.html') 
        elif btn == 'test': 
            return render_template('test.html') 
    else: 
        return render_index() 
 
 
@authorize 
@app.route('/confirm_recording', methods=['POST', 'GET']) 
def confirm_recording(): 
    sleep(1) 
    if recording.status() == 'RUNNING': 
        return render_template('recording.html') 
    else: 
        return render_template('error.html') 
 
 
@authorize 
@app.route('/cancel_recording', methods=['POST', 'GET']) 
def cancel_recording(): 
    Thread(target = recording.stop).start() 
    return render_template('recording.html') 
 
 
@authorize 
@app.route('/pause_recording', methods=['POST', 'GET']) 
def pause_recording(): 
    Thread(target = recording.stop).start() 
    return '200' 
 
 
@authorize 
@app.route('/resume_recording', methods=['POST', 'GET']) 
def resume_recording(): 
    study = default.STUDY 
    site = default.SITE 
    participant = default.PARTICIPANT 
    visit = default.VISIT 
    time =  util.get_curtime() 
    webcams = str(recording.webcams) 
    mics = str(recording.microphones) 
    default.write(study, site, participant, visit, time,  webcams, mics) 
    Thread(target = recording.start).start() 
    return '200' 
 
 
@authorize 
@app.route('/stop_recording', methods=['POST', 'GET']) 
def stop_recording(): 
    Thread(target = recording.stop).start() 
    Thread(target = recording.encrypt).start() 
    return render_index() 
 
 
@app.route('/toggle_sidebar', methods=['GET']) 
def toggle_sidebar(): 
    state = request.args.get('state') 
    device.toggle_sbar(state) 
    return '200' 
 
 
@app.route('/menu', methods=['GET']) 
def select_menu(): 
    btn = request.args.get('btn') 
    menu = { 
        'home' : render_index(), 
        'record' : render_index(), 
        'test' : render_template( 
            'test.html', 
            name = 'test', 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar() 
        ), 
        'config' : render_template( 
            'select_network.html', 
            name = 'network', 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar('config') 
        ), 
        'back' : render_index(), 
        'help' : render_template( 
            'help.html', 
            name = 'overview',
            index = 0,
            sidebar = device.get_sidebar('help') 
        )
    } 
    return menu[btn] 
 
 
@app.route('/config_submenu', methods=['GET']) 
def select_config_submenu(): 
    btn = request.args.get('btn') 
    menu = { 
        'home' : render_index(), 
        'site' : render_template( 
            'select_site.html', 
            name = 'site', 
            configured = device.is_configured(), 
            sites = info.Program.SITES, 
            site = default.SITE, 
            sidebar = device.get_sidebar('config') 
        ), 
        'network' : render_template( 
            'select_network.html', 
            name = 'network', 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar('config') 
        ), 
        'pin' : render_template( 
            'configure_pin.html', 
            name = 'pin', 
            sidebar = device.get_sidebar('config') 
        ), 
        'back' : render_index(), 
        'exit' : render_index() 
    } 
    return menu[btn] 
 
 
@app.route('/get_wifi', methods=['POST', 'GET']) 
def get_wifi(): 
    btn = request.args.get('btn') 
    ssid = request.args.get('ssid') 
    psk = request.args.get('psk') 
    if btn == 'refresh': 
        return render_template( 
            'select_network.html', 
            name = 'network', 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar('config') 
        ) 
    if btn == 'back': 
        return render_index() 
    device.write_wpa(ssid, psk) 
    util.generate_wpa_supplicant(ssid, psk) 
    util.connect()         
    if not util.is_connected():  
        return render_template( 
            'select_network.html', 
            name = 'network', 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar('config') 
        )  
    if device.is_configured(): 
        return render_template( 
            'config_menu.html' 
        ) 
    else: 
        return render_template( 
            'configure_pin.html', 
            name = 'pin', 
            sidebar = device.get_sidebar('config') 
        ) 
 
@app.route('/pin_page', methods=['GET']) 
def pin_page(): 
    return render_template( 
        'configure_pin.html', 
        name = 'pin', 
        sidebar = device.get_sidebar('config') 
    ) 
 
@app.route('/update_connection_status', methods=['GET']) 
def update_connection_status(): 
    global connected 
    if util.is_connected(): 
        connected = True 
        return '200' 
    else: 
        connected = False 
        return '500' 
 
 
# Only used on wifi setup page so that next page load shows wifi true.
# No longer used.
@app.route('/set_connection_status', methods=['GET']) 
def set_connection_status(): 
    status = request.args.get('status') 
    global connected 
    if status == 'False': 
        connected = False 
        return '200' 
    else: 
        connected = True 
        return '200' 
    return 500 
 
 
@app.route('/setup_connection', methods=['POST', 'GET']) 
def setup_connection(): 
    ssid = request.args.get('ssid') 
    psk = request.args.get('psk') 
    try: 
        device.write_wpa(ssid, psk) 
        util.generate_wpa_supplicant(ssid, psk) 
        util.connect() 
        return '200' 
    except: 
        return '500' 
 

# Just used for testing
@app.route('/get_connection_variable', methods=['GET']) 
def check_connection(): 
    global connected 
    print(connected) 
    if connected: 
        return '200' 
    else: 
        return '500' 


@app.route('/check_upload_status', methods=['GET']) 
def check_upload_status():
    global upload_status
    if util.check_upload(): 
        upload_status = True
        return '200'
    else:
        upload_status = False
        return '500'


@app.route('/get_site', methods=['GET']) 
def get_site(): 
    btn = request.args.get('btn') 
    site = request.args.get('site') 
    device.write_site(site) 
    if btn == 'back': 
        return render_template( 
            'welcome.html' 
        ) 
    elif btn == 'continue': 
        return render_template( 
            'select_network.html', 
            name = 'network', 
            ssid = device.SSID, 
            ssids = util.get_network_ssids(), 
            sidebar = device.get_sidebar('config') 
        )


@app.route('/help_config_submenu', methods=['GET']) 
def select_help_submenu(): 
    btn = request.args.get('btn')
    index = 0
    menu = { 
        'home' : render_index(), 
        'overview' : render_template( 
            'help.html', 
            name = 'overview',
            index = 0,
            sidebar = device.get_sidebar('help') 
        ), 
        'connections' : render_template( 
            'help.html', 
            name = 'connections',
            index = 1, 
            sidebar = device.get_sidebar('help') 
        ), 
        'network' : render_template( 
            'help.html', 
            name = 'network',
            index = 2,
            sidebar = device.get_sidebar('help') 
        ),
        'auth' : render_template( 
            'help.html', 
            name = 'auth',
            index =3, 
            sidebar = device.get_sidebar('help') 
        ),
        'record' : render_template( 
            'help.html', 
            name = 'record',
            index = 4, 
            sidebar = device.get_sidebar('help') 
        ),
        'upload' : render_template( 
            'help.html', 
            name = 'upload',
            index = 5,
            sidebar = device.get_sidebar('help') 
        ),
        'exit' : render_index() 
    }
    return menu[btn]
 
 
@app.route('/enlighten', methods=['GET']) 
def enlighten(): 
    util.illuminate() 
    return '200' 
 
def main(): 
    app.run() 
 
@app.context_processor 
def inject_variables(): 
    return dict(wifi=connected, uploading = upload_status)

if __name__ == '__main__': 
    main() 
