#!/usr/bin/python3
""" Utility Classes and Functions """

from time import strftime, gmtime
from flask import render_template, request
from mpbc import info
import boto3
import socket
import subprocess as sp
import pyudev


class Recording:
    """
    Simple class for Recording
    """

    def __init__(self):
        """
        """
        self.webcams = []
        self.microphones = []

        devices = sp.Popen(['arecord', '-l'], stdout=sp.PIPE).communicate()[0]
        for device in devices.rstrip().decode().split('\ncard ')[1:]:
            hw, name = device.split(':')[:2]
            if 'C920' in name:
                self.webcams.append(int(hw))
            elif 'AT2020USBi' in name:
                self.microphones.append(int(hw))


    def start(self):
        """
        Starts a recording process
        """
        sp.Popen(['supervisorctl', 'start', 'record'], stdout=sp.PIPE)


    def status(self):
        """
        Returns the status of the recording process
        """
        child = sp.Popen(['supervisorctl', 'status', 'record'], stdout=sp.PIPE)
        status = child.communicate()[0].decode().split()[1]
        return status


    def stop(self):
        """
        Stops the recording process
        """
        sp.Popen(['kill', self.get_pid()], stdout=sp.PIPE)
        sp.Popen(['supervisorctl', 'stop', 'record'], stdout=sp.PIPE)


    def encrypt(self): 
        """
        Packages and encrypts the resultant media files
        """
        sp.Popen(['supervisorctl', 'start', 'encrypt'], stdout=sp.PIPE)


    def get_pid(self):
        """
        Gets and returns ffmpeg pid
        """
        child = sp.Popen(['pidof', 'ffmpeg'], stdout=sp.PIPE)
        return child.communicate()[0].strip()


class PIN:
    """
    Class for storing and checking user PINs
    """

    def __init__(self, pin):
        """
        """
        self.correct_pin = False
        self.entered_pin = None
        self.pin = pin #change to import pin from file
        self.attempt_counter = 5

    def check_pin(self, entered_pin):
        """
        Checks if the entered pin is the correct pin
        """
        self.entered_pin = entered_pin
        if self.attempt_counter == 0:
            self.correct_pin = False
        elif entered_pin == self.pin:
            self.correct_pin = True
            self.reset_counter()
        else:
            self.correct_pin = False
            self.attempt_counter -= 1
        return self.correct_pin

    def reset_counter(self):
        self.attempt_counter = 5


class USB:
    """
    Class for USB utilities
    """
    def __init__(self):
        """
        """
        self.context = pyudev.Context()
        self.monitor = pyudev.Monitor.from_netlink(self.context)
        self.monitor.filter_by(subsystem='usb')
#        self.data = 

    def is_storage_device_attached(self):
        for device in self.context.list_devices(subsystem='block', DEVTYPE='partition'):
            if device.get('ID_FS_TYPE') == 'ext4':
                return True
            else:
                return False

    def monitor_devices(self):
        for device in iter(monitor.poll, None):
            if device.action == 'add' and device.get('ID_MODEL') is not None:
                if device.get('ID_USB_DRIVER') == 'usb-storage':
                    return device.get('DEVNAME')


    def mount_drive(self, device_name):
        sp.Popen(['sudo', 'mount', device_name, '/mnt/key'], stdout=sp.PIPE)

    
    def decrypt(self):
        sp.Popen(['supervisorctl', 'status', 'decryptkey'], stdout=sp.PIPE)


    def encrypt(self):
        sp.Popen(['supervisorctl', 'status', 'encryptkey'], stdout=sp.PIPE)



def illuminate(status):
    child = sp.Popen(['sudo', 'rpi-backlight', '--get-brightness'], stdout=sp.PIPE)
    if child.communicate()[0].strip().decode() == '5':
        sp.Popen(['sudo', 'rpi-backlight', '--set-brightness', '100'], stdout=sp.PIPE)
    else:
        sp.Popen(['sudo', 'rpi-backlight', '--set-brightness', '5'], stdout=sp.PIPE)


def get_network_ssids():
    """
    Gets the network SSIDs
    """
    ssids = []
    child = sp.Popen(['iwlist', 'wlan0', 'scan'], stdout=sp.PIPE)
    for line in child.communicate()[0].decode().splitlines():    
        if line.strip().startswith('ESSID'):
            ssid = line.split('"')[1]
            if ssid != 'OutOfService':
                ssids.append(ssid)
    return ssids
        

def generate_wpa_supplicant(ssid, psk):
    """
    Generates a wpa_supplicant.conf file with the passed ssid and psk
    """
    print(ssid, psk)
    
    #Assures the password is greater than 7 characters    
    if len(psk) > 7:
        child = sp.Popen(['wpa_passphrase', ssid, psk], stdout=sp.PIPE)
        with open('/etc/wpa_supplicant/wpa_supplicant.conf', 'a') as wpa:
            for line in child.communicate()[0].decode().splitlines():
                if line.strip().startswith('#'):
                   continue
                line+='\n'
                wpa.write(line)

def check_upload():
    c = sp.Popen(['supervisorctl', 'status', 'upload'], stdout=sp.PIPE)
    s = ' '.join(c.communicate()[0].decode().strip().split()).split(' ')[1]
    if s == 'RUNNING': 
        return True
    else:
        return False

def connect():
    """
    Establish a network interface
    """
    sp.Popen(['supervisorctl', 'start', 'connect'], stdout=sp.PIPE)


def is_connected():
  try:
    # see if we can resolve the host name -- tells us if there is
    # a DNS listening
    host = socket.gethostbyname("google.com")
    # connect to the host -- tells us if the host is actually
    # reachable
    s = socket.create_connection((host, 80), 2)
    s.close()
    return True
  except:
     pass
  return False


def get_curtime():
    return strftime("%Y%m%d-%H%M%S", gmtime())

